#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.BTMM
{
	public class OTF : Indicator
	{
		//Program Vars
		private double myPipSize;
		private int opBarUp,opBarDn;
		private double opPriceUp,opPriceDn;
		
		//User Inputs
		private double boTicks		= 2;
		private double stopTicks	= 4;
		
		//Other Vars
		private bool isOTFup, isOTFdn =false;
		
		//Series
		private Series<double> myStateUp;	//Holds +1 or -1
		private Series<double> myStateDn;	//Holds +1 or -1
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Will Color bars, or plot arrows for OTFs";
				Name										= "OTF";
				Calculate									= Calculate.OnEachTick;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				AddPlot(new Stroke(Brushes.Green, 1), 	PlotStyle.Dot, "OTFup");	
				AddPlot(new Stroke(Brushes.Red, 1), 	PlotStyle.Dot, "OTFdn");
				
				AddPlot(new Stroke(Brushes.Green, 1), 	PlotStyle.Line, "OTFupSTOP");	
				AddPlot(new Stroke(Brushes.Red, 1), 	PlotStyle.Line, "OTFdnSTOP");
				
//				UpColor										= Brushes.DeepSkyBlue;
//				DnColor										= Brushes.Purple;
			}
			else if (State == State.Configure)
			{
			}
			else if (State == State.DataLoaded)
			{			
				#region CODE Set PipSize
				if(Instrument.MasterInstrument.InstrumentType == InstrumentType.Forex) //If forex, divide by 10(to remove pipette)
				{
					if(Instrument.FullName.ToString().Contains("JPY") )		myPipSize = 0.01;
					else													myPipSize = 0.0001;	
				}
				else
					myPipSize = TickSize;
				#endregion
//				ClearOutputWindow();
				
				myStateUp = new Series<double>(this, MaximumBarsLookBack.Infinite);
				myStateDn =	new Series<double>(this, MaximumBarsLookBack.Infinite);
//				myDoubleSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
			}
		}

		protected override void OnBarUpdate()
		{
			if(CurrentBar < 5)	return;
			
			if(Bars.IsFirstBarOfSession && IsFirstTickOfBar)
			{
//				opBar =CurrentBar;	opPrice =Open[0];
			}
			
		//Up Side	
			//If Bars 2 and 3 are LOCATED in the correct position
			bool bar2upOk =false;
			if( High[1] >= High[2] + boTicks*myPipSize && Low[1] >= Low[2] ) 
				bar2upOk =true;
			
			bool bar3upOk =false;
			if( Low[0] >= Low[1] -stopTicks*myPipSize) 
				bar3upOk =true;
			
			//Enter Pattern, end of bar 3
			if(bar2upOk && bar3upOk && !isOTFup) 
			{
//				Draw.TriangleUp(this, "tag1"+CurrentBar, true, 0, Low[0] - myPipSize, Brushes.Green);
				isOTFup = true;	
				OTFupSTOP[1] =Low[1] - stopTicks*myPipSize; //1bar extension(left edge)
			}
		
			//Exit up OTF
			if(isOTFup)
			{
				if(Low[0] < Low[1] - stopTicks*myPipSize)
			 	{
//					Draw.TriangleDown(this, "tag2"+CurrentBar, true, 0, Low[0] - myPipSize, Brushes.Green); 
					isOTFup = false;
					OTFupSTOP[0] =Low[1] - stopTicks*myPipSize; //1bar extension(right edge)
					PlotBrushes[0][0] = Brushes.Transparent;
			 	}
			}
			
		//Down Side	
			//If Bars 2 and 3 are LOCATED in the correct position
			bool bar2dnOk =false;
			if( Low[1] <= Low[2] - boTicks*myPipSize && High[1] <= High[2] ) 
				bar2dnOk =true;
			
			bool bar3dnOk =false;
			if( High[0] <= High[1] +stopTicks*myPipSize) 
				bar3dnOk =true;
			
			//Enter Pattern, end of bar 3
			if(bar2dnOk && bar3dnOk && !isOTFdn) 
			{
//				Draw.TriangleDown(this, "tag3"+CurrentBar, true, 0, High[0] + myPipSize, Brushes.Red);
				isOTFdn = true;	
				OTFdnSTOP[1] =High[1] + stopTicks*myPipSize; //1bar extension(left edge)
			}
		
			//Exit down OTF
			if(isOTFdn)
			{
				if(High[0] > High[1] + stopTicks*myPipSize)
			 	{
//					Draw.TriangleUp(this, "tag4"+CurrentBar, true, 0, High[0] + myPipSize, Brushes.Red); 
					isOTFdn = false;
					OTFdnSTOP[0] =High[1] + stopTicks*myPipSize; //1bar extension(right edge)
					PlotBrushes[1][0] = Brushes.Transparent;
			 	}
			}
			
		//Set Flags
			myStateUp[0] =0; myStateDn[0] =0;
			
			if(isOTFup)
			{
				if(OTFup[1] ==0 )
					opPriceUp =Close[0];
				
				OTFup[0] = opPriceUp +0*myPipSize;
				OTFupSTOP[0] =Low[1] - stopTicks*myPipSize;	
				myStateUp[0] =1;
			}

			if(isOTFdn)
			{
				if(OTFdn[1] ==0 )
					opPriceDn =Close[0];
				
				OTFdn[0] = opPriceDn -0*myPipSize;	
				OTFdnSTOP[0] =High[1] + stopTicks*myPipSize;	
				myStateDn[0] =-1;
			}			

			
		}	//End OnBarUpdate()
		
		protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{
			int rightMostBar = ChartBars.ToIndex; 					//Note:First bar on chart is 0 
			int leftMostBar = ChartBars.FromIndex; 					//Note:First bar on chart is 0 

			SharpDX.Direct2D1.SolidColorBrush solidColorBrush;

			for (int i=leftMostBar; i <= rightMostBar; i++) //Start at necessary List position
			{
			 	int	vOffset	= 10;

				float currentX = chartControl.GetXByBarIndex(ChartBars, i);

				int ellipseRadiusY = chartControl.GetBarPaintWidth(chartControl.BarsArray[0])/2;
				int ellipseRadiusX = chartControl.GetBarPaintWidth(chartControl.BarsArray[0])/2;		


				if(myStateUp.GetValueAt(i) == +1)
				{
					SharpDX.Vector2 center = new SharpDX.Vector2(currentX, ChartPanel.Y + (ChartPanel.H) -vOffset -20);
					SharpDX.Direct2D1.Ellipse ellipse = new SharpDX.Direct2D1.Ellipse(center, ellipseRadiusX, ellipseRadiusY);
					
					solidColorBrush = new SharpDX.Direct2D1.SolidColorBrush(RenderTarget, SharpDX.Color.Green);
					RenderTarget.FillEllipse(ellipse, solidColorBrush);
					solidColorBrush.Dispose();
				}
				if(myStateDn.GetValueAt(i) == -1)
				{
					SharpDX.Vector2 center = new SharpDX.Vector2(currentX, ChartPanel.Y + (ChartPanel.H) -vOffset -10);
					SharpDX.Direct2D1.Ellipse ellipse = new SharpDX.Direct2D1.Ellipse(center, ellipseRadiusX, ellipseRadiusY);
					
					solidColorBrush = new SharpDX.Direct2D1.SolidColorBrush(RenderTarget, SharpDX.Color.Red);
					RenderTarget.FillEllipse(ellipse, solidColorBrush);
					solidColorBrush.Dispose();
				}
				if(myStateUp.GetValueAt(i) == 0)
				{
					SharpDX.Vector2 center = new SharpDX.Vector2(currentX, ChartPanel.Y + (ChartPanel.H) -vOffset -20);
					SharpDX.Direct2D1.Ellipse ellipse = new SharpDX.Direct2D1.Ellipse(center, ellipseRadiusX, ellipseRadiusY);
					
					solidColorBrush = new SharpDX.Direct2D1.SolidColorBrush(RenderTarget, SharpDX.Color.Transparent);
					RenderTarget.FillEllipse(ellipse, solidColorBrush);		
					solidColorBrush.Dispose();
				}
				if(myStateDn.GetValueAt(i) == 0)
				{
					SharpDX.Vector2 center = new SharpDX.Vector2(currentX, ChartPanel.Y + (ChartPanel.H) -vOffset -10);
					SharpDX.Direct2D1.Ellipse ellipse = new SharpDX.Direct2D1.Ellipse(center, ellipseRadiusX, ellipseRadiusY);
					
					solidColorBrush = new SharpDX.Direct2D1.SolidColorBrush(RenderTarget, SharpDX.Color.Transparent);
					RenderTarget.FillEllipse(ellipse, solidColorBrush);		
					solidColorBrush.Dispose();
				}

//				Print(Time[0] + "  " + leftMostBar+ "  " + myState.GetValueAt(i) );
				
			}
			
			
			
			//Clean Up
//			solidColorBrush.Dispose();
			
			//Allow onRender+
			base.OnRender(chartControl, chartScale);
		}
		
		#region Properties
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(ResourceType = typeof(Custom.Resource), Name ="BO Ticks", Description = "", GroupName = "Parameters", Order = 1)]
		public double BoTicks
		{	
            get { return boTicks; }
            set { boTicks = value; }
		}
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(ResourceType = typeof(Custom.Resource), Name ="Stop Ticks", Description = "", GroupName = "Parameters", Order = 2)]
		public double StopTicks
		{	
            get { return stopTicks; }
            set { stopTicks = value; }
		}
		
	//--------------------------	
//		[NinjaScriptProperty]
//		[XmlIgnore]
//		[Display(Name="UpColor", Order=3, GroupName="Parameters")]
//		public Brush UpColor
//		{ get; set; }

//		[Browsable(false)]
//		public string UpColorSerializable
//		{
//			get { return Serialize.BrushToString(UpColor); }
//			set { UpColor = Serialize.StringToBrush(value); }
//		}			

//		[NinjaScriptProperty]
//		[XmlIgnore]
//		[Display(Name="DnColor", Order=4, GroupName="Parameters")]
//		public Brush DnColor
//		{ get; set; }

//		[Browsable(false)]
//		public string DnColorSerializable
//		{
//			get { return Serialize.BrushToString(DnColor); }
//			set { DnColor = Serialize.StringToBrush(value); }
//		}	
		
		#endregion
		
		#region Plots
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> OTFup
		{
			get { return Values[0]; }
		}
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> OTFdn
		{
			get { return Values[1]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> OTFupSTOP
		{
			get { return Values[2]; }
		}
		
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> OTFdnSTOP
		{
			get { return Values[3]; }
		}
		
		#endregion
		
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private BTMM.OTF[] cacheOTF;
		public BTMM.OTF OTF(double boTicks, double stopTicks)
		{
			return OTF(Input, boTicks, stopTicks);
		}

		public BTMM.OTF OTF(ISeries<double> input, double boTicks, double stopTicks)
		{
			if (cacheOTF != null)
				for (int idx = 0; idx < cacheOTF.Length; idx++)
					if (cacheOTF[idx] != null && cacheOTF[idx].BoTicks == boTicks && cacheOTF[idx].StopTicks == stopTicks && cacheOTF[idx].EqualsInput(input))
						return cacheOTF[idx];
			return CacheIndicator<BTMM.OTF>(new BTMM.OTF(){ BoTicks = boTicks, StopTicks = stopTicks }, input, ref cacheOTF);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.BTMM.OTF OTF(double boTicks, double stopTicks)
		{
			return indicator.OTF(Input, boTicks, stopTicks);
		}

		public Indicators.BTMM.OTF OTF(ISeries<double> input , double boTicks, double stopTicks)
		{
			return indicator.OTF(input, boTicks, stopTicks);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.BTMM.OTF OTF(double boTicks, double stopTicks)
		{
			return indicator.OTF(Input, boTicks, stopTicks);
		}

		public Indicators.BTMM.OTF OTF(ISeries<double> input , double boTicks, double stopTicks)
		{
			return indicator.OTF(input, boTicks, stopTicks);
		}
	}
}

#endregion
